--[[

	Martin Karer / Sezz, 2010-2011
	Modified Critline Basic Beta 10 by gwiz665 (and all the previous Critline developers).
	Removed all buttons and other indicators, only the splash frame is left...
	
	Use /clreset to reset the database!

	Changelog:
	2011-07-02:
		[*] 4.2 COMBAT_LOG_EVENT_UNFILTERED changes

	2011-05-23:
		[*] Completely switched to spell IDs

	2011-18-05:
		[*] Removed even more obselete code...

	2011-05-16:
		[*] 4.1 COMBAT_LOG_EVENT_UNFILTERED changes

	2010-09-13/2010-09-14:
		[:] Cataclysm Update
		[*] Removed more obselete code/features...
			Features left:
			- Record damage, heal and pet
			- Splash screen
		[*] Lowered DB size/memory usage (maybe replace spell names with IDs as spells are scaling now?)

	2010-01-30:
		[+] New defaults
		[*] Removed a lot of stuff I don't need

--]]

-----------------------------------------------------------------------------
-- Configuration
-----------------------------------------------------------------------------
local colorSpell = { 1, 1, 0, 1 };
local colorAmount = { 1, 1, 1, 1 };
-----------------------------------------------------------------------------
-- There's nothing else here you could modify.
-- REALLY!
-----------------------------------------------------------------------------
local CL_HIT_NORM		= 1;
local CL_HIT_CRIT		= 2;
local CL_TREE_DMG		= 1;
local CL_TREE_HEAL		= 2;
local CL_TREE_PET		= 3;
local CL_SPELLID_MELEE	= 1; -- The real spell#1 is not used and won't occur, so it should be safe to use that...

local Critline_OutOfControl = false;
local Critline_MindControlled = false;

local UnitGUID = UnitGUID;
-----------------------------------------------------------------------------
local Print = function(message)
	DEFAULT_CHAT_FRAME:AddMessage(message);
end
-----------------------------------------------------------------------------
local Critline_Reset = function()
	CLB_SORTED_DB = {};
	Critline_Initialize();
	Print("Critline Basic data purged.");
end
-----------------------------------------------------------------------------
-- Splash
-----------------------------------------------------------------------------
local CLB_DisplayNewRecord = function(spellName, amount, isCritical)
	CritlineSplashFrame:Clear()

	if (isCritical) then
		CritlineSplashFrame:AddMessage(strupper(TEXT_MODE_A_STRING_RESULT_CRITICAL), 1, 0, 0, 1);
	end
	CritlineSplashFrame:AddMessage(amount, unpack(colorAmount));
	CritlineSplashFrame:AddMessage(format("New %s Record!", spellName), unpack(colorSpell));
end
-----------------------------------------------------------------------------
function CritlineSplashFrame_OnLoad(self)
	self:SetSpacing(5);
	self:SetScale(1.5)
end
-----------------------------------------------------------------------------
-- Initialize
-----------------------------------------------------------------------------
Critline_Initialize = function()
	-- Database
	if (CLB_SORTED_DB == nil) then
		CLB_SORTED_DB = {};
	end
	if (CLB_SORTED_DB[CL_TREE_DMG] == nil) then
		CLB_SORTED_DB[CL_TREE_DMG] = {};
	end
	if (CLB_SORTED_DB[CL_TREE_HEAL] == nil) then
		CLB_SORTED_DB[CL_TREE_HEAL] = {};
	end
	if (CLB_SORTED_DB[CL_TREE_PET] == nil) then
		CLB_SORTED_DB[CL_TREE_PET] = {};
	end	

	CritlineSplashFrame:SetTimeVisible(2);
end
-----------------------------------------------------------------------------
-- Combat log handling
-----------------------------------------------------------------------------
local CLB_HitHandle = function(spellName, spellId, spellType, iscrit, amount)
	if (amount == nil) then	return;	end
	
	if (CLB_SORTED_DB == nil) then
		Critline_Initialize();
	end

	local hitType = CL_HIT_NORM;
	if (iscrit) then
		hitType = CL_HIT_CRIT;
	end

	if (CLB_SORTED_DB[spellType] == nil) then
		CLB_SORTED_DB[spellType] = {};
	end
	
	if (not CLB_SORTED_DB[spellType][spellId]) then
		CLB_SORTED_DB[spellType][spellId] = { [CL_HIT_NORM] = 0, [CL_HIT_CRIT] = 0 };
	end

	if (CLB_SORTED_DB[spellType][spellId][hitType] < amount ) then
		CLB_SORTED_DB[spellType][spellId][hitType] = amount;
		CLB_DisplayNewRecord(spellName, amount, iscrit);
	end
end
-----------------------------------------------------------------------------
-- Events
-----------------------------------------------------------------------------
function Critline_OnEvent(self, event, ...)
	if (event == "COMBAT_LOG_EVENT_UNFILTERED") then
		local timestamp, eventType, _, sourceGUID, sourceName, _, _, _, destName = ...
		local spellType = CL_TREE_DMG;

		-- we seem to get events with standard arguments equal to nil, so they need to be ignored
		if (timestamp == nil) or (eventType == nil) then			
			return;
		end

		--if we don't have a destName (who we hit or healed) and we don't have a sourceName(us or our pets) then we leave
		if destName == nil and sourceName == nil then			
			return;
		end
		
		local myGUID = UnitGUID("player");
		local myPetGUID = UnitGUID("pet");
		
		--if sourceGUID  is not us or our pet, we leave
		if sourceGUID ~= myGUID then
			if myPetGUID then
				if sourceGUID ~= myPetGUID then					
					return;
				end
			else				
				return;
			end
		end
		
		if sourceGUID == myPetGUID then
			spellType = CL_TREE_PET;
		end

		if Critline_MindControlled then
			-- critline basic always suppress MC events			
			return;
		end
		
		if (eventType == "SPELL_CAST_START") or (eventType == "SPELL_CAST_FAILED") or (eventType == "SPELL_CAST_SUCCESS") or (eventType == "SWING_DAMAGE") then
			if Critline_OutOfControl then
				--if we are out of control and casting spells and/or attacking, we are mind controlled
				Critline_MindControlled = true;
			end
		end
		
		-- Healing
		if ((eventType == "SPELL_HEAL") or (eventType == "SPELL_PERIODIC_HEAL")) then
			spellType = CL_TREE_HEAL;
		end

		-- Melee Damage
		if (eventType == "SWING_DAMAGE") then
			local amount, _, _, _, _, _, critical = select(12, ...)
			if (amount and (amount == 0)) then return; end
			CLB_HitHandle("Melee", CL_SPELLID_MELEE, spellType, critical, amount); 

		-- Spell/Ranged Damage/Healing
		elseif ((eventType == "RANGE_DAMAGE") or (eventType == "SPELL_DAMAGE") or (eventType == "SPELL_PERIODIC_DAMAGE") or (eventType == "DAMAGE_SPLIT") or (eventType == "DAMAGE_SHIELD") or (eventType == "SPELL_HEAL") or (eventType == "SPELL_PERIODIC_HEAL")) then
			local spellId, spellName, _, amount, _, _, _, _, _, critical = select(12, ...)
			if (amount and (amount == 0)) then return; end
			CLB_HitHandle(spellName, spellId, spellType, critical, amount);
		end

	elseif (event == "ADDON_LOADED") then
		local addonName = ...;
		if (addonName == "sCritline") then
			Critline_Initialize();
			CritlineSplashFrame:EnableMouse(false)
			CritlineSplashFrame:Clear();

			self:UnregisterEvent(event);
			self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
			self:RegisterEvent("PLAYER_CONTROL_LOST");
			self:RegisterEvent("PLAYER_CONTROL_GAINED");
		end

	elseif (event == "PLAYER_CONTROL_LOST") then
		--this covers fear, taxi, mind control, and others
		Critline_OutOfControl = true;

	elseif (event == "PLAYER_CONTROL_GAINED") then
		Critline_OutOfControl = false;
		Critline_MindControlled = false;

	end
end
-----------------------------------------------------------------------------
function Critline_OnLoad(self)
	self:RegisterEvent("ADDON_LOADED");
	SlashCmdList["CRITLINERESET"] = Critline_Reset
	SLASH_CRITLINERESET1 = "/clreset";
end
-----------------------------------------------------------------------------
